<?php
if (!defined('DC_RC_PATH')) { return; }

/* Add behavior callback, will be used for all types of posts (standard, page, galery item, ...) */
$core->addBehavior('coreAfterPostContentFormat',array('imgWidthHeightAdder','afterPostFormat'));

/* Add behavior for the checkbox in the admin preferences */
$core->addBehavior('adminBlogPreferencesForm',array('imgWidthHeightAdder','adminBlogPreferencesForm'));
$core->addBehavior('adminBeforeBlogSettingsUpdate',array('imgWidthHeightAdder','adminBeforeBlogSettingsUpdate'));

class imgWidthHeightAdder {
	protected static $post_imgs = array();
	protected static $post_imgs_reg = '#(?:<img[^>]+src="(.+?)".*?>)#msu';
	
	public static function afterPostFormat($ref) {
		$post_excerpt_xhtml = $ref['excerpt_xhtml'];
		$post_content_xhtml = $ref['content_xhtml'];
		
		self::updatePostEntries(array(
			'excerpt_xhtml' => &$post_excerpt_xhtml,
			'content_xhtml' => &$post_content_xhtml
		));
		
		if ($post_excerpt_xhtml != $ref['excerpt_xhtml'] || $post_content_xhtml != $ref['content_xhtml']) {
			$ref['excerpt_xhtml'] = $post_excerpt_xhtml;
			$ref['content_xhtml'] = $post_content_xhtml;
		}
	}

	public static function updatePostEntries($ref) {
		global $core;
		if (@is_array($ref)) {
			/* Add image sizes for excerpt (XHTML) */
			if (isset($ref['excerpt_xhtml'])) {
				$excerpt = &$ref['excerpt_xhtml'];
				if ($excerpt) {
					// Find every image
					if (preg_match_all(self::$post_imgs_reg,$excerpt,$m) > 0)
					{
						foreach ($m[1] as $i => $v) {
							self::$post_imgs[$v] = array(
								'w' => null,
								'h' => null
							);
						}

						// Lookup the size for each image found
						$media = new iwahaMedia($core);
						$media->getMediaItems(self::$post_imgs);

						// Add size information to every img tag
						$excerpt = preg_replace_callback(self::$post_imgs_reg, array('self', 'photoSize'), $excerpt);
					}
				}
			}

			/* Add image sizes for content (XHTML) */
			if (isset($ref['content_xhtml'])) {
				$content = &$ref['content_xhtml'];
				if ($content) {
					// Find every image
					if (preg_match_all(self::$post_imgs_reg,$content,$m) > 0)
					{
						foreach ($m[1] as $i => $v) {
							self::$post_imgs[$v] = array(
								'w' => null,
								'h' => null
							);
						}

						// Lookup the size for each image found
						$media = new iwahaMedia($core);
						$media->getMediaItems(self::$post_imgs);

						// Add size information to every img tag
						$content = preg_replace_callback(self::$post_imgs_reg, array('self', 'photoSize'), $content);
					}
				}
			}
		}
	}

	protected static function photoSize($match)
	{
		if (!self::$post_imgs[$match[1]]['w']) {
			return $match[0];
		}

		if (strpos($match[0], 'width=') !== FALSE || strpos($match[0], 'height=') !== FALSE) {
			return $match[0];
		}

		$img = self::$post_imgs[$match[1]];
		$width = $img['w'];
		$height = $img['h'];

		$s = preg_replace('#(<img[^>]+src="[^"]+?".*?)/>#msu', '$1'.' height="'.$height.'" width="'.$width.'" />', $match[0]);
		return $s;
	}

	public static function adminBlogPreferencesForm(&$core,&$settings)
	{
		echo
		'<fieldset><legend>IWaHa (Img Width and Height adder)</legend>'.
		'<p><label class="classic">'.
		form::checkbox('iwaha_enabled','1',$settings->iwaha->iwaha_enabled).
		__('Add width and height attributes to images in posts.').'</label></p>'.
		'</fieldset>';
	}
	
	public static function adminBeforeBlogSettingsUpdate(&$settings)
	{
		$settings->addNameSpace('iwaha');
		$settings->iwaha->put('iwaha_enabled',!empty($_POST['iwaha_enabled']),'boolean');
	}

}

class iwahaMedia extends dcMedia
{
	public function getMediaItems(&$post_imgs)
	{
		$paths = array_keys($post_imgs);
		
		foreach ($paths as $v) {
			// Extract the file name for the image
			$file_name = preg_replace('/^'.preg_quote($this->core->blog->settings->system->public_url,'/').'(\/?)/','',$v);

			// Find the file system path to the image
			$f = new fileItem($this->root.'/'.$file_name,$this->root,$this->root_url);

			// Query the image sizes
			$s = getimagesize($f->file);
			$post_imgs[$v]['w'] = $s[0];
			$post_imgs[$v]['h'] = $s[1];
		}
	}
}

?>
